@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,26 +23,30 @@ require_once dirname(__FILE__).'/include/config.inc.php';
 
 $page['title'] = _('Configuration of trigger displaying options');
 $page['file'] = 'adm.triggerdisplayoptions.php';
+$page['scripts'] = ['colorpicker.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 $fields = [
-	'problem_unack_color' =>	[T_ZBX_CLR, O_OPT, null, null, 'isset({update})',
+	'custom_color' =>			[T_ZBX_INT, O_OPT, null, IN('1'), null, _('Use custom event status colours')],
+	'problem_unack_color' =>	[T_ZBX_CLR, O_OPT, null, null, 'isset({update}) && isset({custom_color})',
 		_('Unacknowledged PROBLEM events')
 	],
-	'problem_ack_color' =>		[T_ZBX_CLR, O_OPT, null, null, 'isset({update})',
+	'problem_ack_color' =>		[T_ZBX_CLR, O_OPT, null, null, 'isset({update}) && isset({custom_color})',
 		_('Acknowledged PROBLEM events')
 	],
-	'ok_unack_color' =>			[T_ZBX_CLR, O_OPT, null, null, 'isset({update})', _('Unacknowledged OK events')],
-	'ok_ack_color' =>			[T_ZBX_CLR, O_OPT, null, null, 'isset({update})', _('Acknowledged OK events')],
+	'ok_unack_color' =>			[T_ZBX_CLR, O_OPT, null, null, 'isset({update}) && isset({custom_color})',
+		_('Unacknowledged RESOLVED events')
+	],
+	'ok_ack_color' =>			[T_ZBX_CLR, O_OPT, null, null, 'isset({update}) && isset({custom_color})',
+		_('Acknowledged RESOLVED events')
+	],
 	'problem_unack_style' =>	[T_ZBX_INT, O_OPT, null, IN('1'), null, _('Blinking')],
 	'problem_ack_style' =>		[T_ZBX_INT, O_OPT, null, IN('1'), null, _('Blinking')],
 	'ok_unack_style' =>			[T_ZBX_INT, O_OPT, null, IN('1'), null, _('Blinking')],
 	'ok_ack_style' =>			[T_ZBX_INT, O_OPT, null, IN('1'), null, _('Blinking')],
-	'ok_period' =>				[T_ZBX_INT, O_OPT, null, BETWEEN(0, SEC_PER_DAY), 'isset({update})',
-		_('Display OK triggers for')
-	],
-	'blink_period' =>			[T_ZBX_INT, O_OPT, null, BETWEEN(0, SEC_PER_DAY), 'isset({update})',
+	'ok_period' =>				[T_ZBX_STR, O_OPT, null, null, 'isset({update})', _('Display OK triggers for')],
+	'blink_period' =>			[T_ZBX_STR, O_OPT, null, null, 'isset({update})',
 		_('On status change triggers blink for')
 	],
 	// actions
@@ -55,19 +59,25 @@ check_fields($fields);
  * Actions
  */
 if (hasRequest('update')) {
-	DBstart();
-	$result = update_config([
-		'problem_unack_color' => getRequest('problem_unack_color'),
-		'problem_ack_color' => getRequest('problem_ack_color'),
-		'ok_unack_color' => getRequest('ok_unack_color'),
-		'ok_ack_color' => getRequest('ok_ack_color'),
+	$update_values = [
+		'custom_color' => getRequest('custom_color', EVENT_CUSTOM_COLOR_DISABLED),
 		'problem_unack_style' => getRequest('problem_unack_style', 0),
 		'problem_ack_style' => getRequest('problem_ack_style', 0),
 		'ok_unack_style' => getRequest('ok_unack_style', 0),
 		'ok_ack_style' => getRequest('ok_ack_style', 0),
 		'ok_period' => getRequest('ok_period'),
 		'blink_period' => getRequest('blink_period')
-	]);
+	];
+
+	if ($update_values['custom_color'] == EVENT_CUSTOM_COLOR_ENABLED) {
+		$update_values['problem_unack_color'] = getRequest('problem_unack_color');
+		$update_values['problem_ack_color'] = getRequest('problem_ack_color');
+		$update_values['ok_unack_color'] = getRequest('ok_unack_color');
+		$update_values['ok_ack_color'] = getRequest('ok_ack_color');
+	}
+
+	DBstart();
+	$result = update_config($update_values);
 	$result = DBend($result);
 
 	show_messages($result, _('Configuration updated'), _('Cannot update configuration'));
@@ -81,6 +91,7 @@ $config = select_config();
 // form has been submitted
 if (hasRequest('form_refresh')) {
 	$data = [
+		'custom_color' => getRequest('custom_color', EVENT_CUSTOM_COLOR_DISABLED),
 		'problem_unack_color' => getRequest('problem_unack_color', $config['problem_unack_color']),
 		'problem_ack_color' => getRequest('problem_ack_color', $config['problem_ack_color']),
 		'ok_unack_color' => getRequest('ok_unack_color', $config['ok_unack_color']),
@@ -95,6 +106,7 @@ if (hasRequest('form_refresh')) {
 }
 else {
 	$data = [
+		'custom_color' => $config['custom_color'],
 		'problem_unack_color' => $config['problem_unack_color'],
 		'problem_ack_color' => $config['problem_ack_color'],
 		'ok_unack_color' => $config['ok_unack_color'],
