@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,8 +34,10 @@ $fields = [
 	'druleid' =>		[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({form}) && {form} == "update"'],
 	'name' =>			[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})'],
 	'proxy_hostid' =>	[T_ZBX_INT, O_OPT, null,	DB_ID,		'isset({add}) || isset({update})'],
-	'iprange' =>		[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
-	'delay' =>			[T_ZBX_INT, O_OPT, null,	BETWEEN(1, SEC_PER_WEEK), 'isset({add}) || isset({update})'],
+	'iprange' =>		[T_ZBX_STR, O_OPT, P_CRLF,	null,		'isset({add}) || isset({update})'],
+	'delay' =>			[T_ZBX_TU, O_OPT, P_ALLOW_USER_MACRO, null, 'isset({add}) || isset({update})',
+		_('Update interval')
+	],
 	'status' =>			[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
 	'uniqueness_criteria' => [T_ZBX_STR, O_OPT, null, null,	'isset({add}) || isset({update})', _('Device uniqueness criteria')],
 	'g_druleid' =>		[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
@@ -54,6 +56,11 @@ $fields = [
 	'output' =>			[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
 	'ajaxaction' =>		[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
 	'ajaxdata' =>		[T_ZBX_STR, O_OPT, P_ACT,	null,		null],
+	// filter
+	'filter_set' =>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_rst' =>		[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
+	'filter_name' =>	[T_ZBX_STR, O_OPT, null,	null,		null],
+	'filter_status' =>	[T_ZBX_INT, O_OPT, null,	IN([-1, DRULE_STATUS_ACTIVE, DRULE_STATUS_DISABLED]),		null],
 	// sort and sortorder
 	'sort' =>			[T_ZBX_STR, O_OPT, P_SYS, IN('"name"'),								null],
 	'sortorder' =>		[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
@@ -221,6 +228,15 @@ elseif (hasRequest('action') && getRequest('action') == 'drule.massdelete' && ha
 	show_messages($result, _('Discovery rules deleted'), _('Cannot delete discovery rules'));
 }
 
+if (hasRequest('action') && hasRequest('g_druleid') && !$result) {
+	$drules = API::DRule()->get([
+		'output' => [],
+		'druleids' => getRequest('g_druleid'),
+		'editable' => true
+	]);
+	uncheckTableRows(null, zbx_objectValues($drules, 'druleid'));
+}
+
 /*
  * Display
  */
@@ -249,7 +265,7 @@ if (isset($_REQUEST['form'])) {
 		$data['drule']['proxy_hostid'] = getRequest('proxy_hostid', 0);
 		$data['drule']['name'] = getRequest('name', '');
 		$data['drule']['iprange'] = getRequest('iprange', '192.168.0.1-254');
-		$data['drule']['delay'] = getRequest('delay', SEC_PER_HOUR);
+		$data['drule']['delay'] = getRequest('delay', DB::getDefault('drules', 'delay'));
 		$data['drule']['status'] = getRequest('status', DRULE_STATUS_ACTIVE);
 		$data['drule']['dchecks'] = getRequest('dchecks', []);
 		$data['drule']['nextcheck'] = getRequest('nextcheck', 0);
@@ -286,17 +302,41 @@ else {
 	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
 	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
 
+	// filter
+	if (hasRequest('filter_set')) {
+		CProfile::update('web.discoveryconf.filter_name', getRequest('filter_name', ''), PROFILE_TYPE_STR);
+		CProfile::update('web.discoveryconf.filter_status', getRequest('filter_status', -1), PROFILE_TYPE_INT);
+	}
+	elseif (hasRequest('filter_rst')) {
+		CProfile::delete('web.discoveryconf.filter_name');
+		CProfile::delete('web.discoveryconf.filter_status');
+	}
+
+	$filter = [
+		'name' => CProfile::get('web.discoveryconf.filter_name', ''),
+		'status' => CProfile::get('web.discoveryconf.filter_status', -1)
+	];
+
 	$config = select_config();
 
 	$data = [
 		'sort' => $sortField,
-		'sortorder' => $sortOrder
+		'sortorder' => $sortOrder,
+		'filter' => $filter,
+		'profileIdx' => 'web.discoveryconf.filter',
+		'active_tab' => CProfile::get('web.discoveryconf.filter.active', 1)
 	];
 
 	// get drules
 	$data['drules'] = API::DRule()->get([
 		'output' => ['proxy_hostid', 'name', 'status', 'iprange', 'delay'],
 		'selectDChecks' => ['type'],
+		'search' => [
+			'name' => ($filter['name'] === '') ? null : $filter['name']
+		],
+		'filter' => [
+			'status' => ($filter['status'] == -1) ? null : $filter['status']
+		],
 		'editable' => true,
 		'sortfield' => $sortField,
 		'limit' => $config['search_limit'] + 1
@@ -329,7 +369,7 @@ else {
 	}
 
 	// get paging
-	$data['paging'] = getPagingLine($data['drules'], $sortOrder);
+	$data['paging'] = getPagingLine($data['drules'], $sortOrder, new CUrl('discoveryconf.php'));
 
 	// render view
 	$discoveryView = new CView('configuration.discovery.list', $data);
