@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -373,7 +373,7 @@ class CApiService {
 	protected function createSelectQuery($tableName, array $options) {
 		$sqlParts = $this->createSelectQueryParts($tableName, $this->tableAlias(), $options);
 
-		return $this->createSelectQueryFromParts($sqlParts);
+		return self::createSelectQueryFromParts($sqlParts);
 	}
 
 	/**
@@ -410,6 +410,31 @@ class CApiService {
 		return $sqlParts;
 	}
 
+	/**
+	 * Returns DISTINCT modifier for sql statements with multiple joins.
+	 *
+	 * @param array $sql_parts  An SQL parts array.
+	 *
+	 * @return string
+	 */
+	private static function dbDistinct(array $sql_parts) {
+		$count = count($sql_parts['from']);
+
+		if ($count == 1 && array_key_exists('left_join', $sql_parts)) {
+			foreach ($sql_parts['left_join'] as $left_join) {
+				$r_table = DB::getSchema($left_join['table']);
+
+				// Increase count when table linked by non-unique column.
+				if ($left_join['using'] !== $r_table['key']) {
+					$count++;
+					break;
+				}
+			}
+		}
+
+		return ($count > 1 ? ' DISTINCT' : '');
+	}
+
 	/**
 	 * Creates a SELECT SQL query from the given SQL parts array.
 	 *
@@ -417,17 +442,21 @@ class CApiService {
 	 *
 	 * @return string			The resulting SQL query
 	 */
-	protected function createSelectQueryFromParts(array $sqlParts) {
+	protected static function createSelectQueryFromParts(array $sqlParts) {
 		$sql_left_join = '';
 		if (array_key_exists('left_join', $sqlParts)) {
-			foreach ($sqlParts['left_join'] as $join) {
-				$sql_left_join .= ' LEFT JOIN '.$join['from'].' ON '.$join['on'];
+			$l_table = DB::getSchema($sqlParts['left_table']['table']);
+
+			foreach ($sqlParts['left_join'] as $left_join) {
+				$sql_left_join .= ' LEFT JOIN '.$left_join['table'].' '.$left_join['alias'].
+					' ON '.$sqlParts['left_table']['alias'].'.'.$l_table['key'].
+					'='.$left_join['alias'].'.'.$left_join['using'];
 			}
 
 			// Moving a left table to the end.
-			$left_table = $sqlParts['from'][$sqlParts['left_table']];
-			unset($sqlParts['from'][$sqlParts['left_table']]);
-			$sqlParts['from'][$sqlParts['left_table']] = $left_table;
+			$table_id = $sqlParts['left_table']['table'].' '.$sqlParts['left_table']['alias'];
+			unset($sqlParts['from'][array_search($table_id, $sqlParts['from'])]);
+			$sqlParts['from'][] = $table_id;
 		}
 
 		$sqlSelect = implode(',', array_unique($sqlParts['select']));
@@ -436,7 +465,7 @@ class CApiService {
 		$sqlGroup = empty($sqlParts['group']) ? '' : ' GROUP BY '.implode(',', array_unique($sqlParts['group']));
 		$sqlOrder = empty($sqlParts['order']) ? '' : ' ORDER BY '.implode(',', array_unique($sqlParts['order']));
 
-		return 'SELECT'.zbx_db_distinct($sqlParts).' '.$sqlSelect.
+		return 'SELECT'.self::dbDistinct($sqlParts).' '.$sqlSelect.
 				' FROM '.$sqlFrom.
 				$sql_left_join.
 				$sqlWhere.
@@ -455,12 +484,16 @@ class CApiService {
 	 * @return array		The resulting SQL parts array
 	 */
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
-		$pkFieldId = $this->fieldId($this->pk($tableName), $tableAlias);
+		// If table do not have a primary key, use COUNT(*) to select number of rows.
+		$pk = $this->pk($tableName);
+		$pkFieldId = $this->fieldId($pk, $tableAlias);
 
 		// count
 		if (array_key_exists('countOutput', $options) && $options['countOutput']
 				&& !$this->requiresPostSqlFiltering($options)) {
-			$sqlParts['select'] = ['COUNT(DISTINCT '.$pkFieldId.') AS rowscount'];
+			$sqlParts['select'] = ($pk !== '')
+				? ['COUNT(DISTINCT '.$pkFieldId.') AS rowscount']
+				: ['COUNT(*) AS rowscount'];
 
 			// select columns used by group count
 			if (array_key_exists('groupCount', $options) && $options['groupCount']) {
@@ -472,7 +505,7 @@ class CApiService {
 		// custom output
 		elseif (is_array($options['output'])) {
 			// the pk field must always be included for the API to work properly
-			$sqlParts['select'] = [$pkFieldId];
+			$sqlParts['select'] = ($pk !== '') ? [$pkFieldId] : [];
 			foreach ($options['output'] as $field) {
 				if ($this->hasField($field, $tableName)) {
 					$sqlParts['select'][] = $this->fieldId($field, $tableAlias);
@@ -644,8 +677,8 @@ class CApiService {
 	 * Adds the related objects requested by "select*" options to the resulting object set.
 	 *
 	 * @param array $options
-	 * @param array $result		an object hash with PKs as keys
-
+	 * @param array $result   An object hash with PKs as keys.
+	 *
 	 * @return array mixed
 	 */
 	protected function addRelatedObjects(array $options, array $result) {
