@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 class CHostGroup extends CApiService {
 
-	protected $tableName = 'groups';
+	protected $tableName = 'hstgrp';
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['groupid', 'name'];
 
@@ -37,12 +37,10 @@ class CHostGroup extends CApiService {
 	 */
 	public function get($params) {
 		$result = [];
-		$userType = self::$userData['type'];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
-			'select'	=> ['groups' => 'g.groupid'],
-			'from'		=> ['groups' => 'groups g'],
+			'select'	=> ['hstgrp' => 'g.groupid'],
+			'from'		=> ['hstgrp' => 'hstgrp g'],
 			'where'		=> [],
 			'order'		=> [],
 			'limit'		=> null
@@ -68,14 +66,14 @@ class CHostGroup extends CApiService {
 			'with_monitored_httptests'	=> null,
 			'with_graphs'				=> null,
 			'with_applications'			=> null,
-			'editable'					=> null,
+			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
 			'filter'					=> null,
 			'search'					=> null,
 			'searchByAny'				=> null,
-			'startSearch'				=> null,
-			'excludeSearch'				=> null,
+			'startSearch'				=> false,
+			'excludeSearch'				=> false,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
@@ -83,9 +81,9 @@ class CHostGroup extends CApiService {
 			'selectTemplates'			=> null,
 			'selectGroupDiscovery'		=> null,
 			'selectDiscoveryRule'		=> null,
-			'countOutput'				=> null,
-			'groupCount'				=> null,
-			'preservekeys'				=> null,
+			'countOutput'				=> false,
+			'groupCount'				=> false,
+			'preservekeys'				=> false,
 			'sortfield'					=> '',
 			'sortorder'					=> '',
 			'limit'						=> null,
@@ -94,10 +92,9 @@ class CHostGroup extends CApiService {
 		$options = zbx_array_merge($defOptions, $params);
 
 		// editable + PERMISSION CHECK
-		if ($userType != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-
-			$userGroups = getUserGroupsByUserId($userid);
+			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
 
 			$sqlParts['where'][] = 'EXISTS ('.
 				'SELECT NULL'.
@@ -300,12 +297,12 @@ class CHostGroup extends CApiService {
 
 		// filter
 		if (is_array($options['filter'])) {
-			$this->dbFilter('groups g', $options, $sqlParts);
+			$this->dbFilter('hstgrp g', $options, $sqlParts);
 		}
 
 		// search
 		if (is_array($options['search'])) {
-			zbx_db_search('groups g', $options, $sqlParts);
+			zbx_db_search('hstgrp g', $options, $sqlParts);
 		}
 
 		// limit
@@ -315,10 +312,10 @@ class CHostGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($group = DBfetch($res)) {
-			if (!is_null($options['countOutput'])) {
-				if (!is_null($options['groupCount'])) {
+			if ($options['countOutput']) {
+				if ($options['groupCount']) {
 					$result[] = $group;
 				}
 				else {
@@ -330,7 +327,7 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		if (!is_null($options['countOutput'])) {
+		if ($options['countOutput']) {
 			return $result;
 		}
 
@@ -339,205 +336,181 @@ class CHostGroup extends CApiService {
 		}
 
 		// removing keys (hash -> array)
-		if (is_null($options['preservekeys'])) {
+		if (!$options['preservekeys']) {
 			$result = zbx_cleanHashes($result);
 		}
+
 		return $result;
 	}
 
 	/**
-	 * Create host groups.
+	 * Inherit rights from parent host groups.
 	 *
-	 * @param array $groups array with host group names
-	 * @param array $groups['name']
+	 * @param array  $groups
+	 * @param string $groups[]['groupid']
+	 * @param string $groups[]['name']
+	 */
+	private function inheritRights(array $groups) {
+		$parent_names = [];
+
+		foreach ($groups as $group) {
+			if (($pos = strrpos($group['name'], '/')) === false) {
+				continue;
+			}
+
+			$parent_names[substr($group['name'], 0, $pos)][] = $group['groupid'];
+		}
+
+		if ($parent_names) {
+			$db_parent_groups = DB::select('hstgrp', [
+				'output' => ['groupid', 'name'],
+				'filter' => ['name' => array_keys($parent_names)]
+			]);
+
+			$parent_groupids = [];
+
+			foreach ($db_parent_groups as $db_parent_group) {
+				$parent_groupids[$db_parent_group['groupid']] = $parent_names[$db_parent_group['name']];
+			}
+
+			if ($parent_groupids) {
+				$db_rights = DB::select('rights', [
+					'output' => ['groupid', 'id', 'permission'],
+					'filter' => ['id' => array_keys($parent_groupids)]
+				]);
+
+				$rights = [];
+
+				foreach ($db_rights as $db_right) {
+					foreach ($parent_groupids[$db_right['id']] as $groupid) {
+						$rights[] = [
+							'groupid' => $db_right['groupid'],
+							'permission' => $db_right['permission'],
+							'id' => $groupid
+						];
+					}
+				}
+
+				DB::insertBatch('rights', $rights);
+			}
+		}
+	}
+
+	/**
+	 * Inherit tag filters from parent host groups.
+	 *
+	 * @param array  $groups
+	 * @param string $groups[]['groupid']
+	 * @param string $groups[]['name']
+	 */
+	private function inheritTagFilters(array $groups) {
+		$parent_names = [];
+
+		foreach ($groups as $group) {
+			if (($pos = strrpos($group['name'], '/')) === false) {
+				continue;
+			}
+
+			$parent_names[substr($group['name'], 0, $pos)][] = $group['groupid'];
+		}
+
+		if ($parent_names) {
+			$db_parent_groups = DB::select('hstgrp', [
+				'output' => ['groupid', 'name'],
+				'filter' => ['name' => array_keys($parent_names)]
+			]);
+
+			$parent_groupids = [];
+
+			foreach ($db_parent_groups as $db_parent_group) {
+				$parent_groupids[$db_parent_group['groupid']] = $parent_names[$db_parent_group['name']];
+			}
+
+			if ($parent_groupids) {
+				$db_tag_filters = DB::select('tag_filter', [
+					'output' => ['usrgrpid', 'groupid', 'tag', 'value'],
+					'filter' => ['groupid' => array_keys($parent_groupids)]
+				]);
+
+				$tag_filters = [];
+
+				foreach ($db_tag_filters as $db_tag_filter) {
+					foreach ($parent_groupids[$db_tag_filter['groupid']] as $groupid) {
+						$tag_filters[] = [
+							'usrgrpid' => $db_tag_filter['usrgrpid'],
+							'groupid' => $groupid,
+							'tag' => $db_tag_filter['tag'],
+							'value' => $db_tag_filter['value']
+						];
+					}
+				}
+
+				DB::insertBatch('tag_filter', $tag_filters);
+			}
+		}
+	}
+
+	/**
+	 * @param array  $groups
 	 *
 	 * @return array
 	 */
 	public function create(array $groups) {
-		$groups = zbx_toArray($groups);
-
 		$this->validateCreate($groups);
 
-		$groupids = [];
-		$groupids_with_rights = [];
-		$parent_children = [];
+		$groupids = DB::insertBatch('hstgrp', $groups);
 
-		foreach ($groups as $group) {
-			$new_groupids = DB::insert('groups', [$group]);
-			$groupid = reset($new_groupids);
-			$groupids[] = $groupid;
-
-			$name_parts = explode('/', $group['name']);
-			if (count($name_parts) > 1) {
-				array_pop($name_parts);
-				$parents = $this->get([
-					'output' => ['groupid'],
-					'filter' => ['name' => implode('/', $name_parts)],
-					'limit' => 1
-				]);
-
-				if ($parents) {
-					$parent = reset($parents);
-					$groupids_with_rights[$groupid] = $parent['groupid'];
-					$parent_children[$parent['groupid']][] = $groupid;
-				}
-			}
+		foreach ($groups as $index => &$group) {
+			$group['groupid'] = $groupids[$index];
 		}
+		unset($group);
 
-		if ($groupids_with_rights) {
-			$rights = DBSelect(
-				'SELECT r.groupid, r.permission, r.id'.
-				' FROM rights r'.
-				' WHERE '.dbConditionInt('r.id', $groupids_with_rights)
-			);
+		$this->inheritRights($groups);
+		$this->inheritTagFilters($groups);
 
-			$rights_to_insert = [];
-
-			while ($right = DBfetch($rights)) {
-				foreach ($parent_children[$right['id']] as $parent_child) {
-					$rights_to_insert[] = [
-						'groupid' => $right['groupid'],
-						'permission' => $right['permission'],
-						'id' => $parent_child
-					];
-				}
-			}
-
-			DB::insert('rights', $rights_to_insert);
-		}
+		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_HOST_GROUP, $groups);
 
 		return ['groupids' => $groupids];
 	}
 
 	/**
-	 * Update host groups.
+	 * @param array  $groups
 	 *
-	 * @param array $groups
-	 * @param array $groups[0]['name'], ...
-	 * @param array $groups[0]['groupid'], ...
-	 *
-	 * @return boolean
+	 * @return array
 	 */
 	public function update(array $groups) {
-		$groups = zbx_toArray($groups);
-
-		$db_groups = [];
 		$this->validateUpdate($groups, $db_groups);
 
-		$update = [];
+		$upd_groups = [];
 
 		foreach ($groups as $group) {
-			// Skip update if name has not changed.
-			if (array_key_exists('name', $group) && $group['name'] !== $db_groups[$group['groupid']]['name']) {
-				$update[] = [
+			$db_group = $db_groups[$group['groupid']];
+
+			if (array_key_exists('name', $group) && $group['name'] !== $db_group['name']) {
+				$upd_groups[] = [
 					'values' => ['name' => $group['name']],
 					'where' => ['groupid' => $group['groupid']]
 				];
 			}
 		}
 
-		if ($update) {
-			DB::update('groups', $update);
-		}
+		DB::update('hstgrp', $upd_groups);
+
+		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST_GROUP, $groups, $db_groups);
 
 		return ['groupids' => zbx_objectValues($groups, 'groupid')];
 	}
 
 	/**
-	 * Delete host groups.
-	 *
 	 * @param array $groupids
 	 * @param bool 	$nopermissions
 	 *
+	 * @throws APIException if the input is invalid.
+	 *
 	 * @return array
 	 */
 	public function delete(array $groupids, $nopermissions = false) {
-		if (empty($groupids)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
-		}
-		sort($groupids);
-
-		$delGroups = $this->get([
-			'groupids' => $groupids,
-			'editable' => true,
-			'output' => ['groupid', 'name', 'internal'],
-			'preservekeys' => true,
-			'nopermissions' => $nopermissions
-		]);
-		foreach ($groupids as $groupid) {
-			if (!isset($delGroups[$groupid])) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-			if ($delGroups[$groupid]['internal'] == ZBX_INTERNAL_GROUP) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group "%1$s" is internal and can not be deleted.', $delGroups[$groupid]['name']));
-			}
-		}
-
-		// check if a group is used in a group prototype
-		$groupPrototype = DBFetch(DBselect(
-			'SELECT groupid'.
-			' FROM group_prototype gp'.
-			' WHERE '.dbConditionInt('groupid', $groupids),
-			1
-		));
-		if ($groupPrototype) {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Group "%1$s" cannot be deleted, because it is used by a host prototype.',
-					$delGroups[$groupPrototype['groupid']]['name']
-				)
-			);
-		}
-
-		$dltGroupids = getDeletableHostGroupIds($groupids);
-		if (count($groupids) != count($dltGroupids)) {
-			foreach ($groupids as $groupid) {
-				if (isset($dltGroupids[$groupid])) {
-					continue;
-				}
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group "%1$s" cannot be deleted, because some hosts depend on it.',
-						$delGroups[$groupid]['name']
-					)
-				);
-			}
-		}
-
-		$dbScripts = API::Script()->get([
-			'groupids' => $groupids,
-			'output' => ['scriptid', 'groupid'],
-			'nopermissions' => true
-		]);
-
-		if (!empty($dbScripts)) {
-			foreach ($dbScripts as $script) {
-				if ($script['groupid'] == 0) {
-					continue;
-				}
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
-						$delGroups[$script['groupid']]['name']
-					)
-				);
-			}
-		}
-
-		$corr_condition_group = DBFetch(DBselect(
-			'SELECT cg.groupid'.
-			' FROM corr_condition_group cg'.
-			' WHERE '.dbConditionInt('cg.groupid', $groupids),
-			1
-		));
-
-		if ($corr_condition_group) {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Group "%1$s" cannot be deleted, because it is used in a correlation condition.',
-					$delGroups[$corr_condition_group['groupid']]['name']
-				)
-			);
-		}
+		$this->validateDelete($groupids, $db_groups, $nopermissions);
 
 		// delete screens items
 		$resources = [
@@ -560,18 +533,18 @@ class CHostGroup extends CApiService {
 		// disable actions
 		// actions from conditions
 		$actionids = [];
-		$dbActions = DBselect(
+		$db_actions = DBselect(
 			'SELECT DISTINCT c.actionid'.
 			' FROM conditions c'.
 			' WHERE c.conditiontype='.CONDITION_TYPE_HOST_GROUP.
 				' AND '.dbConditionString('c.value', $groupids)
 		);
-		while ($dbAction = DBfetch($dbActions)) {
-			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
+		while ($db_action = DBfetch($db_actions)) {
+			$actionids[$db_action['actionid']] = $db_action['actionid'];
 		}
 
 		// actions from operations
-		$dbActions = DBselect(
+		$db_actions = DBselect(
 			'SELECT o.actionid'.
 			' FROM operations o,opgroup og'.
 			' WHERE o.operationid=og.operationid AND '.dbConditionInt('og.groupid', $groupids).
@@ -580,8 +553,8 @@ class CHostGroup extends CApiService {
 			' FROM operations o,opcommand_grp ocg'.
 			' WHERE o.operationid=ocg.operationid AND '.dbConditionInt('ocg.groupid', $groupids)
 		);
-		while ($dbAction = DBfetch($dbActions)) {
-			$actionids[$dbAction['actionid']] = $dbAction['actionid'];
+		while ($db_action = DBfetch($db_actions)) {
+			$actionids[$db_action['actionid']] = $db_action['actionid'];
 		}
 
 		if (!empty($actionids)) {
@@ -601,119 +574,64 @@ class CHostGroup extends CApiService {
 
 		// delete action operation groups
 		$operationids = [];
-		$dbOperations = DBselect(
+		$db_operations = DBselect(
 			'SELECT DISTINCT og.operationid'.
 			' FROM opgroup og'.
 			' WHERE '.dbConditionInt('og.groupid', $groupids)
 		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
+		while ($db_operation = DBfetch($db_operations)) {
+			$operationids[$db_operation['operationid']] = $db_operation['operationid'];
 		}
 		DB::delete('opgroup', [
 			'groupid' => $groupids
 		]);
 
 		// delete action operation commands
-		$dbOperations = DBselect(
+		$db_operations = DBselect(
 			'SELECT DISTINCT ocg.operationid'.
 			' FROM opcommand_grp ocg'.
 			' WHERE '.dbConditionInt('ocg.groupid', $groupids)
 		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$operationids[$dbOperation['operationid']] = $dbOperation['operationid'];
+		while ($db_operation = DBfetch($db_operations)) {
+			$operationids[$db_operation['operationid']] = $db_operation['operationid'];
 		}
 		DB::delete('opcommand_grp', [
 			'groupid' => $groupids
 		]);
 
 		// delete empty operations
-		$delOperationids = [];
-		$dbOperations = DBselect(
+		$del_operationids = [];
+		$db_operations = DBselect(
 			'SELECT DISTINCT o.operationid'.
 			' FROM operations o'.
 			' WHERE '.dbConditionInt('o.operationid', $operationids).
 				' AND NOT EXISTS (SELECT NULL FROM opgroup og WHERE o.operationid=og.operationid)'.
 				' AND NOT EXISTS (SELECT NULL FROM opcommand_grp ocg WHERE o.operationid=ocg.operationid)'
 		);
-		while ($dbOperation = DBfetch($dbOperations)) {
-			$delOperationids[$dbOperation['operationid']] = $dbOperation['operationid'];
+		while ($db_operation = DBfetch($db_operations)) {
+			$del_operationids[$db_operation['operationid']] = $db_operation['operationid'];
 		}
 
-		DB::delete('operations', ['operationid' => $delOperationids]);
+		DB::delete('operations', ['operationid' => $del_operationids]);
 
-		DB::delete('groups', ['groupid' => $groupids]);
+		DB::delete('hstgrp', ['groupid' => $groupids]);
 
-		DB::delete('profiles', [
-			'idx' => ['web.dashconf.groups.groupids', 'web.dashconf.groups.subgroupids',
-				'web.dashconf.groups.hide.groupids', 'web.dashconf.groups.hide.subgroupids'
-			],
-			'value_id' => $groupids
-		]);
-
-		// TODO: remove audit
-		foreach ($groupids as $groupid) {
-			add_audit_ext(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_HOST_GROUP, $groupid, $delGroups[$groupid]['name'], 'groups', null, null);
-		}
+		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_HOST_GROUP, $db_groups);
 
 		return ['groupids' => $groupids];
 	}
 
 	/**
-	 * Validates the input parameters for the create() method.
+	 * Check for duplicated host groups.
 	 *
-	 * @param array $groups
+	 * @param array  $names
 	 *
-	 * @throws APIException if the input is invalid.
+	 * @throws APIException  if host group already exists.
 	 */
-	protected function validateCreate(array $groups) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
-		}
-
-		if (!$groups) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
-		}
-
-		$host_group_name_validator = new CHostGroupNameValidator();
-
-		foreach ($groups as $group) {
-			// Validate required fields and check if "name" is not empty.
-			if (!is_array($group)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
-			}
-
-			// Check required parameters.
-			$missing_keys = checkRequiredKeys($group, ['name']);
-			if ($missing_keys) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group is missing parameters: %1$s', implode(', ', $missing_keys))
-				);
-			}
-
-			// Check if host group name is valid.
-			if (!$host_group_name_validator->validate($group['name'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Incorrect value for field "%1$s": %2$s.', 'name', $host_group_name_validator->getError())
-				);
-			}
-
-			$this->checkNoParameters($group, ['internal'], _('Cannot set "%1$s" for host group "%2$s".'),
-				$group['name']
-			);
-		}
-
-		// Check for duplicate names.
-		$duplicate = CArrayHelper::findDuplicate($groups, 'name');
-		if ($duplicate) {
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Duplicate "%1$s" value "%2$s" for host group.', 'name', $duplicate['name'])
-			);
-		}
-
-		// Check if host group already exists.
-		$db_groups = API::getApiService()->select('groups', [
+	private function checkDuplicates(array $names) {
+		$db_groups = DB::select('hstgrp', [
 			'output' => ['name'],
-			'filter' => ['name' => zbx_objectValues($groups, 'name')],
+			'filter' => ['name' => $names],
 			'limit' => 1
 		]);
 
@@ -725,35 +643,153 @@ class CHostGroup extends CApiService {
 	}
 
 	/**
-	 * Validates the input parameters for the update() method.
+	 * @param array $groups
 	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	protected function validateCreate(array &$groups) {
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
+		}
+
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+			'name' =>	['type' => API_HG_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hstgrp', 'name')]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$this->checkDuplicates(zbx_objectValues($groups, 'name'));
+	}
+
+	/**
+	 * Validates if groups can be deleted.
+	 *
+	 * @param array $groupids
+	 * @param array $db_groups
+	 * @param bool $nopermissions
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDelete(array $groupids, array &$db_groups = null, $nopermissions) {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $groupids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$db_groups = $this->get([
+			'output' => ['groupid', 'name', 'internal'],
+			'groupids' => $groupids,
+			'editable' => true,
+			'selectHosts' => ['hostid', 'host'],
+			'selectTemplates' => ['templateid', 'host'],
+			'preservekeys' => true,
+			'nopermissions' => $nopermissions
+		]);
+
+		foreach ($groupids as $groupid) {
+			if (!array_key_exists($groupid, $db_groups)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+			if ($db_groups[$groupid]['internal'] == ZBX_INTERNAL_GROUP) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Host group "%1$s" is internal and can not be deleted.', $db_groups[$groupid]['name'])
+				);
+			}
+		}
+
+		// check if a group is used in a group prototype
+		$group_prototype = DBFetch(DBselect(
+			'SELECT groupid'.
+			' FROM group_prototype gp'.
+			' WHERE '.dbConditionInt('groupid', $groupids),
+			1
+		));
+		if ($group_prototype) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Group "%1$s" cannot be deleted, because it is used by a host prototype.',
+					$db_groups[$group_prototype['groupid']]['name']
+				)
+			);
+		}
+
+		$hosts_to_unlink = [];
+		$templates_to_unlink = [];
+
+		foreach ($db_groups as $db_group) {
+			foreach ($db_group['hosts'] as $host) {
+				$hosts_to_unlink[] = $host;
+			}
+
+			foreach ($db_group['templates'] as $template) {
+				$templates_to_unlink[] = $template;
+			}
+		}
+
+		$this->verifyHostsAndTemplatesAreUnlinkable($hosts_to_unlink, $templates_to_unlink, $groupids);
+
+		$db_scripts = DB::select('scripts', [
+			'output' => ['groupid'],
+			'filter' => ['groupid' => $groupids],
+			'limit' => 1
+		]);
+
+		if ($db_scripts) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
+					$db_groups[$db_scripts[0]['groupid']]['name']
+				)
+			);
+		}
+
+		$corr_condition_group = DBFetch(DBselect(
+			'SELECT cg.groupid'.
+			' FROM corr_condition_group cg'.
+			' WHERE '.dbConditionInt('cg.groupid', $groupids),
+			1
+		));
+
+		if ($corr_condition_group) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Group "%1$s" cannot be deleted, because it is used in a correlation condition.',
+					$db_groups[$corr_condition_group['groupid']]['name']
+				)
+			);
+		}
+
+		$this->validateDeleteCheckMaintenances($groupids);
+	}
+
+	/**
 	 * @param array $groups
 	 * @param array $db_groups
 	 *
 	 * @throws APIException if the input is invalid.
 	 */
-	protected function validateUpdate(array $groups, array &$db_groups) {
-		if (!$groups) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+	protected function validateUpdate(array &$groups, array &$db_groups = null) {
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['groupid'], ['name']], 'fields' => [
+			'groupid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
+			'name' =>		['type' => API_HG_NAME, 'length' => DB::getFieldLength('hstgrp', 'name')]
+		]];
+		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		// Validate given IDs.
-		$this->checkObjectIds($groups, 'groupid',
-			_('No "%1$s" given for group.'),
-			_('Empty group ID.'),
-			_('Incorrect group ID.')
-		);
-
 		// permissions
 		$db_groups = $this->get([
-			'output' => ['groupid', 'flags', 'name'],
+			'output' => ['groupid', 'name', 'flags'],
 			'groupids' => zbx_objectValues($groups, 'groupid'),
 			'editable' => true,
 			'preservekeys' => true
 		]);
 
-		$check_names = [];
-		$host_group_name_validator = new CHostGroupNameValidator();
+		$update_discovered_validator = new CUpdateDiscoveredValidator([
+			'messageAllowed' => _('Cannot update a discovered host group.')
+		]);
+
+		$names = [];
 
 		foreach ($groups as $group) {
 			if (!array_key_exists($group['groupid'], $db_groups)) {
@@ -762,55 +798,17 @@ class CHostGroup extends CApiService {
 				);
 			}
 
-			// Check if host group name is valid.
-			if (array_key_exists('name', $group)) {
-				if (!$host_group_name_validator->validate($group['name'])) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value for field "%1$s": %2$s.', 'name', $host_group_name_validator->getError())
-					);
-				}
+			$db_group = $db_groups[$group['groupid']];
 
-				if ($db_groups[$group['groupid']]['name'] !== $group['name']) {
-					$check_names[] = $group;
-				}
+			$this->checkPartialValidator($group, $update_discovered_validator, $db_group);
+
+			if (array_key_exists('name', $group) && $group['name'] !== $db_group['name']) {
+				$names[] = $group['name'];
 			}
 		}
 
-		if ($check_names) {
-			// Check for duplicate names.
-			$duplicate = CArrayHelper::findDuplicate($check_names, 'name');
-			if ($duplicate) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Duplicate "%1$s" value "%2$s" for host group.', 'name', $duplicate['name'])
-				);
-			}
-
-			// Check if group already exists.
-			$db_group_names = API::getApiService()->select('groups', [
-				'output' => ['groupid', 'name'],
-				'filter' => ['name' => zbx_objectValues($check_names, 'name')]
-			]);
-			$db_group_names = zbx_toHash($db_group_names, 'name');
-
-			foreach ($check_names as $group) {
-				if (array_key_exists($group['name'], $db_group_names)
-						&& bccomp($db_group_names[$group['name']]['groupid'], $group['groupid']) != 0) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Host group "%1$s" already exists.', $group['name']));
-				}
-			}
-		}
-
-		$update_discovered_validator = new CUpdateDiscoveredValidator([
-			'messageAllowed' => _('Cannot update a discovered host group.')
-		]);
-
-		foreach ($groups as $group) {
-			$this->checkNoParameters($group, ['internal'], _('Cannot update "%1$s" for host group "%2$s".'),
-				array_key_exists('name', $group) ? $group['name'] : $db_groups[$group['groupid']]['name']
-			);
-
-			// Cannot update discovered host groups.
-			$this->checkPartialValidator($group, $update_discovered_validator, $db_groups[$group['groupid']]);
+		if ($names) {
+			$this->checkDuplicates($names);
 		}
 	}
 
@@ -904,6 +902,10 @@ class CHostGroup extends CApiService {
 	 *								removed from
 	 */
 	public function massUpdate(array $data) {
+		if (!array_key_exists('groups', $data) || !is_array($data['groups'])) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Field "%1$s" is mandatory.', 'groups'));
+		}
+
 		$data['groups'] = zbx_toArray($data['groups']);
 		$data['hosts'] = isset($data['hosts']) ? zbx_toArray($data['hosts']) : [];
 		$data['templates'] = isset($data['templates']) ? zbx_toArray($data['templates']) : [];
@@ -1022,10 +1024,18 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		if ($groupIdsToAdd && !$this->isWritable($groupIdsToAdd)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS,
-				_('No permissions to referred object or it does not exist!')
-			);
+		if ($groupIdsToAdd) {
+			$count = $this->get([
+				'countOutput' => true,
+				'groupids' => $groupIdsToAdd,
+				'editable' => true
+			]);
+
+			if ($count != count($groupIdsToAdd)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
 		}
 	}
 
@@ -1048,9 +1058,9 @@ class CHostGroup extends CApiService {
 
 		$dbGroups = $this->get([
 			'output' => ['groupid'],
-			'selectHosts' => ['hostid'],
-			'selectTemplates' => ['templateid'],
-			'groupids' => $groupIds
+			'groupids' => $groupIds,
+			'selectHosts' => ['hostid', 'host'],
+			'selectTemplates' => ['templateid', 'host']
 		]);
 
 		if (!$dbGroups) {
@@ -1070,15 +1080,6 @@ class CHostGroup extends CApiService {
 		 */
 		$objectIds = [];
 
-		// Collect both given host and template IDs.
-		$currentObjectIds = array_merge($hostIds, $templateIds);
-		if ($currentObjectIds) {
-			$currentObjectIds = array_combine($currentObjectIds, $currentObjectIds);
-		}
-
-		// Collect both host and template IDs that also belong to given groups, but are not given in parameters.
-		$objectIdsToRemove = [];
-
 		/*
 		 * New or existing hosts have been passed in parameters. First check write permissions to hosts
 		 * and if hosts are not discovered. Then check if groups should be added and/or removed from given hosts.
@@ -1117,19 +1118,6 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		/*
-		 * Some existing hosts may not have been passed in parameters. In this case check what other hosts the groups
-		 * have and if those hosts can be removed from given groups. This will also validate those hosts permissions and
-		 * make sure hosts have at least one group left.
-		 */
-		foreach ($dbGroups as $dbGroup) {
-			foreach ($dbGroup['hosts'] as $dbHost) {
-				if (!isset($currentObjectIds[$dbHost['hostid']])) {
-					$objectIdsToRemove[$dbHost['hostid']] = $dbHost['hostid'];
-				}
-			}
-		}
-
 		/*
 		 * New or existing templates have been passed in parameters. First check write permissions to templates.
 		 * Then check if groups should be added and/or removed from given templates.
@@ -1164,27 +1152,22 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		/*
-		 * Some existing templates may not have been passed in parameters. In this case check what other templates the
-		 * groups have and if those templates can be removed from given groups. This will also validate those template
-		 * permissions and make sure templates have at least one group left.
-		 */
-		foreach ($dbGroups as $dbGroup) {
-			foreach ($dbGroup['templates'] as $dbTemplate) {
-				if (!isset($currentObjectIds[$dbTemplate['templateid']])) {
-					$objectIdsToRemove[$dbTemplate['templateid']] = $dbTemplate['templateid'];
-				}
-			}
-		}
-
 		// Continue to check new, existing or removable groups for given hosts and templates.
-		$groupIdsToUpdate = array_merge($groupIdsToAdd, $groupIdsToRemove);
+		$groupIdsToUpdate = $groupIdsToAdd + $groupIdsToRemove;
 
 		// Validate write permissions only to changed (added/removed) groups for given hosts and templates.
-		if ($groupIdsToUpdate && !$this->isWritable($groupIdsToUpdate)) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS,
-				_('No permissions to referred object or it does not exist!')
-			);
+		if ($groupIdsToUpdate) {
+			$count = $this->get([
+				'countOutput' => true,
+				'groupids' => $groupIdsToUpdate,
+				'editable' => true
+			]);
+
+			if ($count != count($groupIdsToUpdate)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
 		}
 
 		// Check if groups can be removed from given hosts and templates. Only check if no groups are added.
@@ -1196,14 +1179,26 @@ class CHostGroup extends CApiService {
 			}
 		}
 
-		// Check the other way around - if other existing hosts and templates from those groups are left without groups.
-		if ($objectIdsToRemove) {
-			$unlinkableObjectIds = getUnlinkableHostIds($groupIds, $objectIdsToRemove);
+		$hosts_to_unlink = [];
+		$templates_to_unlink = [];
+		$hostIds = array_flip($hostIds);
+		$templateIds = array_flip($templateIds);
 
-			if (count($objectIdsToRemove) != count($unlinkableObjectIds)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('One of the objects is left without a host group.'));
+		foreach ($dbGroups as $group) {
+			foreach ($group['hosts'] as $host) {
+				if (!array_key_exists($host['hostid'], $hostIds)) {
+					$hosts_to_unlink[] = $host;
+				}
+			}
+
+			foreach ($group['templates'] as $template) {
+				if (!array_key_exists($template['templateid'], $templateIds)) {
+					$templates_to_unlink[] = $template;
+				}
 			}
 		}
+
+		$this->verifyHostsAndTemplatesAreUnlinkable($hosts_to_unlink, $templates_to_unlink, $groupIds);
 	}
 
 	/**
@@ -1222,11 +1217,12 @@ class CHostGroup extends CApiService {
 		$groupIdsToRemove = [];
 		$hostIds = isset($data['hostids']) ? $data['hostids'] : [];
 		$templateIds = isset($data['templateids']) ? $data['templateids'] : [];
-		$objectIds = [];
+		$hosts_to_unlink = [];
+		$templates_to_unlink = [];
 
 		if ($hostIds) {
 			$dbHosts = API::Host()->get([
-				'output' => ['hostid'],
+				'output' => ['hostid', 'host'],
 				'selectGroups' => ['groupid'],
 				'hostids' => $hostIds,
 				'editable' => true,
@@ -1246,7 +1242,7 @@ class CHostGroup extends CApiService {
 				$hostGroupIdsToRemove = array_intersect($data['groupids'], $oldGroupIds);
 
 				if ($hostGroupIdsToRemove) {
-					$objectIds[] = $dbHost['hostid'];
+					$hosts_to_unlink[] = $dbHost;
 
 					foreach ($hostGroupIdsToRemove as $groupId) {
 						$groupIdsToRemove[$groupId] = $groupId;
@@ -1257,7 +1253,7 @@ class CHostGroup extends CApiService {
 
 		if ($templateIds) {
 			$dbTemplates = API::Template()->get([
-				'output' => ['templateid'],
+				'output' => ['templateid', 'host'],
 				'selectGroups' => ['groupid'],
 				'templateids' => $templateIds,
 				'editable' => true,
@@ -1273,7 +1269,7 @@ class CHostGroup extends CApiService {
 				$templateGroupIdsToRemove = array_intersect($data['groupids'], $oldGroupIds);
 
 				if ($templateGroupIdsToRemove) {
-					$objectIds[] = $dbTemplate['templateid'];
+					$templates_to_unlink[] = $dbTemplate;
 
 					foreach ($templateGroupIdsToRemove as $groupId) {
 						$groupIdsToRemove[$groupId] = $groupId;
@@ -1283,21 +1279,20 @@ class CHostGroup extends CApiService {
 		}
 
 		if ($groupIdsToRemove) {
-			if (!$this->isWritable($groupIdsToRemove)) {
+			$count = $this->get([
+				'countOutput' => true,
+				'groupids' => $groupIdsToRemove,
+				'editable' => true
+			]);
+
+			if ($count != count($groupIdsToRemove)) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS,
 					_('No permissions to referred object or it does not exist!')
 				);
 			}
-
-			// check if host group can be removed from given hosts and templates leaving them with at least one more host group
-			$unlinkableObjectIds = getUnlinkableHostIds($groupIdsToRemove, $objectIds);
-
-			if (count($objectIds) != count($unlinkableObjectIds)) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_('One of the objects is left without a host group.')
-				);
-			}
 		}
+
+		$this->verifyHostsAndTemplatesAreUnlinkable($hosts_to_unlink, $templates_to_unlink, $groupIdsToRemove);
 	}
 
 	/**
@@ -1318,56 +1313,6 @@ class CHostGroup extends CApiService {
 		}
 	}
 
-	/**
-	 * Check if user has read permissions for host groups.
-	 *
-	 * @param array $ids
-	 *
-	 * @return bool
-	 */
-	public function isReadable(array $ids) {
-		if (!is_array($ids)) {
-			return false;
-		}
-		if (empty($ids)) {
-			return true;
-		}
-
-		$ids = array_unique($ids);
-
-		$count = $this->get([
-			'groupids' => $ids,
-			'countOutput' => true
-		]);
-		return count($ids) == $count;
-	}
-
-	/**
-	 * Check if user has write permissions for host groups.
-	 *
-	 * @param array $ids
-	 *
-	 * @return bool
-	 */
-	public function isWritable(array $ids) {
-		if (!is_array($ids)) {
-			return false;
-		}
-		if (empty($ids)) {
-			return true;
-		}
-
-		$ids = array_unique($ids);
-
-		$count = $this->get([
-			'groupids' => $ids,
-			'editable' => true,
-			'countOutput' => true
-		]);
-
-		return count($ids) == $count;
-	}
-
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
@@ -1396,12 +1341,9 @@ class CHostGroup extends CApiService {
 				]);
 				$hosts = zbx_toHash($hosts, 'groupid');
 				foreach ($result as $groupid => $group) {
-					if (isset($hosts[$groupid])) {
-						$result[$groupid]['hosts'] = $hosts[$groupid]['rowscount'];
-					}
-					else {
-						$result[$groupid]['hosts'] = 0;
-					}
+					$result[$groupid]['hosts'] = array_key_exists($groupid, $hosts)
+						? $hosts[$groupid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1428,12 +1370,9 @@ class CHostGroup extends CApiService {
 				]);
 				$hosts = zbx_toHash($hosts, 'groupid');
 				foreach ($result as $groupid => $group) {
-					if (isset($hosts[$groupid])) {
-						$result[$groupid]['templates'] = $hosts[$groupid]['rowscount'];
-					}
-					else {
-						$result[$groupid]['templates'] = 0;
-					}
+					$result[$groupid]['templates'] = array_key_exists($groupid, $hosts)
+						? $hosts[$groupid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -1475,4 +1414,83 @@ class CHostGroup extends CApiService {
 
 		return $result;
 	}
+
+	/**
+	 * Verify that hosts and templates are unlinkable from groups.
+	 *
+	 * @param array     $hosts
+	 * @param integer   $hosts[]['hostid']
+	 * @param string    $hosts[]['host']
+	 * @param array     $templates
+	 * @param integer   $templates[]['templateid']
+	 * @param string    $templates[]['host']
+	 * @param array     $groupids
+	 */
+	protected function verifyHostsAndTemplatesAreUnlinkable(array $hosts, array $templates, array $groupids) {
+		$objectids = [];
+		$host_names = [];
+		$template_names = [];
+
+		foreach ($hosts as $host) {
+			$objectids[] = $host['hostid'];
+			$host_names[$host['hostid']] = $host['host'];
+		}
+
+		foreach ($templates as $template) {
+			$objectids[] = $template['templateid'];
+			$template_names[$template['templateid']] = $template['host'];
+		}
+
+		if ($objectids && $groupids) {
+			$not_unlinkable_objectids = array_diff($objectids, getUnlinkableHostIds($groupids, $objectids));
+
+			if ($not_unlinkable_objectids) {
+				$objectid = reset($not_unlinkable_objectids);
+
+				if (array_key_exists($objectid, $host_names)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Host "%1$s" cannot be without host group.', $host_names[$objectid])
+					);
+				}
+
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Template "%1$s" cannot be without host group.', $template_names[$objectid])
+				);
+			}
+		}
+	}
+
+	/**
+	 * Validates if host groups may be deleted, due to maintenance constrain.
+	 *
+	 * @throws APIException if a constrain failed
+	 *
+	 * @param array $groupids
+	 */
+	protected function validateDeleteCheckMaintenances(array $groupids) {
+		$maintenance = DBfetch(DBselect(
+			'SELECT m.name'.
+			' FROM maintenances m'.
+			' WHERE NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM maintenances_groups mg'.
+				' WHERE m.maintenanceid=mg.maintenanceid'.
+					' AND '.dbConditionInt('mg.groupid', $groupids, true).
+			')'.
+				' AND NOT EXISTS ('.
+					'SELECT NULL'.
+					' FROM maintenances_hosts mh'.
+					' WHERE m.maintenanceid=mh.maintenanceid'.
+				')'
+		));
+
+		if ($maintenance) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _n(
+				'Cannot delete host group because maintenance "%1$s" must contain at least one host or host group.',
+				'Cannot delete selected host groups because maintenance "%1$s" must contain at least one host or host group.',
+				$maintenance['name'],
+				count($groupids)
+			));
+		}
+	}
 }
