@@ -812,16 +812,19 @@ class CHostInterface extends CApiService {
 			]);
 			$host = reset($host);
 
-			foreach ($db_interfaces as $db_interface) {
-				if (bccomp($db_interface['hostid'], $host['hostid']) == 0) {
-					$type = $db_interface['type'];
-					break;
+			if ($host) {
+				foreach ($db_interfaces as $db_interface) {
+					if (bccomp($db_interface['hostid'], $host['hostid']) == 0) {
+						$type = $db_interface['type'];
+						break;
+					}
 				}
-			}
 
-			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('No default interface for "%1$s" type on "%2$s".', hostInterfaceTypeNumToName($type), $host['name'])
-			);
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s(
+					'No default interface for "%1$s" type on "%2$s".', hostInterfaceTypeNumToName($type), $host['name']
+				));
+			}
+			// Otherwise it's not a host. Could be a Proxy.
 		}
 
 		$interface_count = [];
@@ -892,9 +895,12 @@ class CHostInterface extends CApiService {
 					]);
 					$host = reset($host);
 
-					self::exception(ZBX_API_ERROR_PARAMETERS,_s('No default interface for "%1$s" type on "%2$s".',
-						hostInterfaceTypeNumToName($type), $host['name']
-					));
+					if ($host) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,_s('No default interface for "%1$s" type on "%2$s".',
+							hostInterfaceTypeNumToName($type), $host['name']
+						));
+					}
+					// Otherwise it's not a host. Could be a Proxy.
 				}
 
 				if ($counters['main'] > 1) {
