@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 class CAjaxResponse {
 
 	private $_result = true;
-	private $_data = array();
-	private $_errors = array();
+	private $_data = [];
+	private $_errors = [];
 
 	public function __construct($data = null) {
 		if ($data !== null) {
@@ -38,11 +38,10 @@ class CAjaxResponse {
 	 * Add error to ajax response. All errors are returned as array in 'errors' part of response.
 	 *
 	 * @param string $error error text
-	 * @return void
 	 */
 	public function error($error) {
 		$this->_result = false;
-		$this->_errors[] = array('error' => $error);
+		$this->_errors[] = ['error' => $error];
 	}
 
 	/**
@@ -50,7 +49,6 @@ class CAjaxResponse {
 	 * If any error was added previously, this method does nothing.
 	 *
 	 * @param array $data
-	 * @return void
 	 */
 	public function success(array $data) {
 		if ($this->_result) {
@@ -60,17 +58,15 @@ class CAjaxResponse {
 
 	/**
 	 * Output ajax response. If any error was added, 'result' is false, otherwise true.
-	 *
-	 * @return void
 	 */
 	public function send() {
 		$json = new CJson();
 
 		if ($this->_result) {
-			echo $json->encode(array('result' => true, 'data' => $this->_data));
+			echo $json->encode(['result' => true, 'data' => $this->_data]);
 		}
 		else {
-			echo $json->encode(array('result' => false, 'errors' => $this->_errors));
+			echo $json->encode(['result' => false, 'errors' => $this->_errors]);
 		}
 	}
 }
