@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,8 +30,7 @@ class CConfigFile {
 		ZBX_DB_DB2 => true,
 		ZBX_DB_MYSQL => true,
 		ZBX_DB_ORACLE => true,
-		ZBX_DB_POSTGRESQL => true,
-		ZBX_DB_SQLITE3 => true
+		ZBX_DB_POSTGRESQL => true
 	];
 
 	public $configFile = null;
@@ -77,10 +76,11 @@ class CConfigFile {
 			);
 		}
 
-		if (!array_key_exists($DB['TYPE'], CFrontendSetup::getSupportedDatabases())) {
-			self::exception(
-				'DB type "'.$DB['TYPE'].'" is not supported by current setup. Possible values '.
-				implode(', ', array_keys(CFrontendSetup::getSupportedDatabases())).'.'
+		$php_supported_db = array_keys(CFrontendSetup::getSupportedDatabases());
+
+		if (!in_array($DB['TYPE'], $php_supported_db)) {
+			self::exception('DB type "'.$DB['TYPE'].'" is not supported by current setup.'.
+				($php_supported_db ? ' Possible values '.implode(', ', $php_supported_db).'.' : '')
 			);
 		}
 
@@ -145,7 +145,12 @@ class CConfigFile {
 
 			$this->check();
 
-			if (!file_put_contents($this->configFile, $this->getString())) {
+			if (file_put_contents($this->configFile, $this->getString())) {
+				if (!chmod($this->configFile, 0600)) {
+					self::exception(_('Unable to change configuration file permissions to 0600.'));
+				}
+			}
+			else {
 				if (file_exists($this->configFile)) {
 					if (file_get_contents($this->configFile) !== $this->getString()) {
 						self::exception(_('Unable to overwrite the existing configuration file.'));
@@ -185,7 +190,6 @@ $ZBX_SERVER_PORT = \''.addcslashes($this->config['ZBX_SERVER_PORT'], "'\\").'\';
 $ZBX_SERVER_NAME = \''.addcslashes($this->config['ZBX_SERVER_NAME'], "'\\").'\';
 
 $IMAGE_FORMAT_DEFAULT = IMAGE_FORMAT_PNG;
-?>
 ';
 	}
 
