@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,11 @@
 
 class CPieGraphDraw extends CGraphDraw {
 
+	const DEFAULT_HEADER_PADDING_TOP = 30;
+
+	const GRAPH_WIDTH_MIN = 20;
+	const GRAPH_HEIGHT_MIN = 20;
+
 	public function __construct($type = GRAPH_TYPE_PIE) {
 		parent::__construct($type);
 		$this->background = false;
@@ -35,7 +40,7 @@ class CPieGraphDraw extends CGraphDraw {
 	/* PRE CONFIG: ADD / SET / APPLY
 	/********************************************************************************************************/
 	public function addItem($itemid, $calc_fnc = CALC_FNC_AVG, $color = null, $type = null) {
-		$items = CMacrosResolverHelper::resolveItemNames(array(get_item_by_itemid($itemid)));
+		$items = CMacrosResolverHelper::resolveItemNames([get_item_by_itemid($itemid)]);
 
 		$this->items[$this->num] = reset($items);
 
@@ -120,48 +125,49 @@ class CPieGraphDraw extends CGraphDraw {
 		$y+= round($count * sin(deg2rad($anglemid)));
 		$x+= round($count * cos(deg2rad($anglemid)));
 
-		return array($x, $y);
+		return [$x, $y];
 	}
 
 	protected function calcExplodedRadius($sizeX, $sizeY, $count) {
 		$count *= $this->exploderad * 2;
 		$sizeX -= $count;
 		$sizeY -= $count;
-		return array($sizeX, $sizeY);
+		return [$sizeX, $sizeY];
 	}
 
 	protected function calc3DAngle($sizeX, $sizeY) {
 		$sizeY *= GRAPH_3D_ANGLE / 90;
-		return array($sizeX, round($sizeY));
+		return [$sizeX, round($sizeY)];
 	}
 
 	protected function selectData() {
-		$this->data = array();
-		$now = time(null);
+		$this->data = [];
+		$now = time();
 
 		if (isset($this->stime)) {
 			$this->from_time = $this->stime;
 			$this->to_time = $this->stime + $this->period;
 		}
 		else {
-			$this->to_time = $now - SEC_PER_HOUR * $this->from;
+			$this->to_time = $now;
 			$this->from_time = $this->to_time - $this->period;
 		}
 
 		$strvaluelength = 0; // we need to know how long in px will be our legend
 
 		// fetch values for items with the "last" function
-		$lastValueItems = array();
+		$lastValueItems = [];
 		foreach ($this->items as $item) {
 			if ($item['calc_fnc'] == CALC_FNC_LST) {
 				$lastValueItems[] = $item;
 			}
 		}
 		if ($lastValueItems) {
-			$history = Manager::History()->getLast($lastValueItems);
+			$history = Manager::History()->getLastValues($lastValueItems);
 		}
 
 		$config = select_config();
+		$items = [];
 
 		for ($i = 0; $i < $this->num; $i++) {
 			$item = get_item_by_itemid($this->items[$i]['itemid']);
@@ -169,60 +175,48 @@ class CPieGraphDraw extends CGraphDraw {
 			$from_time = $this->from_time;
 			$to_time = $this->to_time;
 
-			$sql_arr = array();
+			$to_resolve = [];
 
-			// override item history setting with housekeeping settings
+			// Override item history setting with housekeeping settings, if they are enabled in config.
 			if ($config['hk_history_global']) {
-				$item['history'] = $config['hk_history'];
-			}
-
-			$trendsEnabled = $config['hk_trends_global'] ? ($config['hk_trends'] > 0) : ($item['trends'] > 0);
-
-			if (!$trendsEnabled || (($item['history'] * SEC_PER_DAY) > (time() - ($from_time + $this->period / 2)))) {
-				$this->dataFrom = 'history';
-
-				array_push($sql_arr,
-					'SELECT h.itemid,'.
-						'AVG(h.value) AS avg,MIN(h.value) AS min,'.
-						'MAX(h.value) AS max,MAX(h.clock) AS clock'.
-					' FROM history h'.
-					' WHERE h.itemid='.zbx_dbstr($this->items[$i]['itemid']).
-						' AND h.clock>='.zbx_dbstr($from_time).
-						' AND h.clock<='.zbx_dbstr($to_time).
-					' GROUP BY h.itemid'
-					,
-					'SELECT hu.itemid,'.
-						'AVG(hu.value) AS avg,MIN(hu.value) AS min,'.
-						'MAX(hu.value) AS max,MAX(hu.clock) AS clock'.
-					' FROM history_uint hu'.
-					' WHERE hu.itemid='.zbx_dbstr($this->items[$i]['itemid']).
-						' AND hu.clock>='.zbx_dbstr($from_time).
-						' AND hu.clock<='.zbx_dbstr($to_time).
-					' GROUP BY hu.itemid'
-				);
+				$item['history'] = timeUnitToSeconds($config['hk_history']);
 			}
 			else {
-				$this->dataFrom = 'trends';
+				$to_resolve[] = 'history';
+			}
 
-				array_push($sql_arr,
-					'SELECT t.itemid,'.
-						'AVG(t.value_avg) AS avg,MIN(t.value_min) AS min,'.
-						'MAX(t.value_max) AS max,MAX(t.clock) AS clock'.
-					' FROM trends t'.
-					' WHERE t.itemid='.zbx_dbstr($this->items[$i]['itemid']).
-						' AND t.clock>='.zbx_dbstr($from_time).
-						' AND t.clock<='.zbx_dbstr($to_time).
-					' GROUP BY t.itemid'
-					,
-					'SELECT t.itemid,'.
-						'AVG(t.value_avg) AS avg,MIN(t.value_min) AS min,'.
-						'MAX(t.value_max) AS max,MAX(t.clock) AS clock'.
-					' FROM trends_uint t'.
-					' WHERE t.itemid='.zbx_dbstr($this->items[$i]['itemid']).
-						' AND t.clock>='.zbx_dbstr($from_time).
-						' AND t.clock<='.zbx_dbstr($to_time).
-					' GROUP BY t.itemid'
-				);
+			if ($config['hk_trends_global']) {
+				$item['trends'] = timeUnitToSeconds($config['hk_trends']);
+			}
+			else {
+				$to_resolve[] = 'trends';
+			}
+
+			// Otherwise, resolve user macro and parse the string. If successful, convert to seconds.
+			if ($to_resolve) {
+				$item = CMacrosResolverHelper::resolveTimeUnitMacros([$item], $to_resolve)[0];
+
+				$simple_interval_parser = new CSimpleIntervalParser();
+
+				if (!$config['hk_history_global']) {
+					if ($simple_interval_parser->parse($item['history']) != CParser::PARSE_SUCCESS) {
+						show_error_message(_s('Incorrect value for field "%1$s": %2$s.', 'history',
+							_('invalid history storage period')
+						));
+						exit;
+					}
+					$item['history'] = timeUnitToSeconds($item['history']);
+				}
+
+				if (!$config['hk_trends_global']) {
+					if ($simple_interval_parser->parse($item['trends']) != CParser::PARSE_SUCCESS) {
+						show_error_message(_s('Incorrect value for field "%1$s": %2$s.', 'trends',
+							_('invalid trend storage period')
+						));
+						exit;
+					}
+					$item['trends'] = timeUnitToSeconds($item['trends']);
+				}
 			}
 
 			$this->data[$this->items[$i]['itemid']][$type]['last'] = isset($history[$item['itemid']])
@@ -231,15 +225,31 @@ class CPieGraphDraw extends CGraphDraw {
 			$this->data[$this->items[$i]['itemid']][$type]['shift_max'] = 0;
 			$this->data[$this->items[$i]['itemid']][$type]['shift_avg'] = 0;
 
-			foreach ($sql_arr as $sql) {
-				$result = DBselect($sql);
-				while ($row = DBfetch($result)) {
-					$this->data[$this->items[$i]['itemid']][$type]['min'] = $row['min'];
-					$this->data[$this->items[$i]['itemid']][$type]['max'] = $row['max'];
-					$this->data[$this->items[$i]['itemid']][$type]['avg'] = $row['avg'];
-					$this->data[$this->items[$i]['itemid']][$type]['clock'] = $row['clock'];
+			$item['source'] = ($item['trends'] == 0 || ($item['history'] > time() - ($from_time + $this->period / 2)))
+				? 'history'
+				: 'trends';
+			$items[] = $item;
+		}
+
+		$results = Manager::History()->getGraphAggregation($items, $from_time, $to_time);
+		$i = 0;
+
+		foreach ($items as $item) {
+			if (array_key_exists($item['itemid'], $results)) {
+				$result = $results[$item['itemid']];
+				$this->dataFrom = $result['source'];
+				$type = $this->items[$i]['calc_type'];
+
+				foreach ($result['data'] as $row) {
+					$this->data[$item['itemid']][$type]['min'] = $row['min'];
+					$this->data[$item['itemid']][$type]['max'] = $row['max'];
+					$this->data[$item['itemid']][$type]['avg'] = $row['avg'];
+					$this->data[$item['itemid']][$type]['clock'] = $row['clock'];
 				}
-				unset($row);
+				unset($result);
+			}
+			else {
+				$this->dataFrom = $item['source'];
 			}
 
 			switch ($this->items[$i]['calc_fnc']) {
@@ -257,9 +267,9 @@ class CPieGraphDraw extends CGraphDraw {
 					$fncName = 'avg';
 			}
 
-			$item_value = empty($this->data[$this->items[$i]['itemid']][$type][$fncName])
+			$item_value = empty($this->data[$item['itemid']][$type][$fncName])
 				? 0
-				: abs($this->data[$this->items[$i]['itemid']][$type][$fncName]);
+				: abs($this->data[$item['itemid']][$type][$fncName]);
 
 			if ($type == GRAPH_ITEM_SUM) {
 				$this->background = $i;
@@ -268,11 +278,12 @@ class CPieGraphDraw extends CGraphDraw {
 
 			$this->sum += $item_value;
 
-			$convertedUnit = strlen(convert_units(array(
+			$convertedUnit = strlen(convert_units([
 				'value' => $item_value,
-				'units' => $this->items[$i]['units']
-			)));
+				'units' => $item['units']
+			]));
 			$strvaluelength = max($strvaluelength, $convertedUnit);
+			$i++;
 		}
 
 		if (isset($graph_sum)) {
@@ -302,6 +313,7 @@ class CPieGraphDraw extends CGraphDraw {
 
 		// display items
 		$i = 0;
+		$top_padding = $this->with_vertical_padding ? 10 : -(static::DEFAULT_TOP_BOTTOM_PADDING / 2);
 
 		foreach ($this->items as $item) {
 			$color = $this->getColor($item['color'], 0);
@@ -332,10 +344,10 @@ class CPieGraphDraw extends CGraphDraw {
 				$proc = ($this->sum == 0) ? 0 : ($dataValue * 100) / $this->sum;
 
 				$strValue = sprintf(_('Value').': %s ('.(round($proc) != round($proc, 2) ? '%0.2f' : '%0.0f').'%%)',
-					convert_units(array(
+					convert_units([
 						'value' => $dataValue,
 						'units' => $this->items[$i]['units']
-					)),
+					]),
 					$proc
 				);
 
@@ -395,9 +407,9 @@ class CPieGraphDraw extends CGraphDraw {
 			imagefilledrectangle(
 				$this->im,
 				$shiftX - 10,
-				$this->shiftY + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 14 * $i,
 				$shiftX,
-				$this->shiftY + 10 + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 10 + 14 * $i,
 				$color
 			);
 
@@ -405,9 +417,9 @@ class CPieGraphDraw extends CGraphDraw {
 			imagerectangle(
 				$this->im,
 				$shiftX - 10,
-				$this->shiftY + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 14 * $i,
 				$shiftX,
-				$this->shiftY + 10 + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 10 + 14 * $i,
 				$this->GetColor('Black No Alpha')
 			);
 
@@ -417,7 +429,7 @@ class CPieGraphDraw extends CGraphDraw {
 				$fontSize,
 				0,
 				$shiftX + 5,
-				$this->shiftY + 10 + 14 * $i + 10,
+				$this->shiftY + $top_padding + 14 * $i + 10,
 				$this->getColor($this->graphtheme['textcolor'], 0),
 				$strValue
 			);
@@ -444,7 +456,7 @@ class CPieGraphDraw extends CGraphDraw {
 		}
 
 		if ($sum <= 0) {
-			$values = array(0 => 1);
+			$values = [0 => 1];
 			$sum = 1;
 			$isEmptyData = true;
 		}
@@ -458,19 +470,21 @@ class CPieGraphDraw extends CGraphDraw {
 		if ($this->type == GRAPH_TYPE_EXPLODED) {
 			list($sizeX, $sizeY) = $this->calcExplodedRadius($sizeX, $sizeY, count($values));
 		}
-		else {
-			$sizeX = (int) $sizeX * 0.95;
-			$sizeY = (int) $sizeY * 0.95;
-		}
 
 		$xc = $x = (int) $this->sizeX / 2 + $this->shiftXleft;
 		$yc = $y = (int) $this->sizeY / 2 + $this->shiftY;
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -519,7 +533,7 @@ class CPieGraphDraw extends CGraphDraw {
 		}
 
 		if ($sum <= 0) {
-			$values = array(0 => 1);
+			$values = [0 => 1];
 			$sum = 1;
 			$isEmptyData = true;
 		}
@@ -544,9 +558,15 @@ class CPieGraphDraw extends CGraphDraw {
 		// bottom angle line
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -554,6 +574,7 @@ class CPieGraphDraw extends CGraphDraw {
 			if ($this->type == GRAPH_TYPE_3D_EXPLODED) {
 				list($x, $y) = $this->calcExplodedCenter($anglestart, $angleend, $xc, $yc, count($values));
 			}
+
 			imagefilledarc(
 				$this->im,
 				$x,
@@ -583,7 +604,12 @@ class CPieGraphDraw extends CGraphDraw {
 		for ($i = $this->graphheight3d; $i > 0; $i--) {
 			$anglestart = 0;
 			$angleend = 0;
+
 			foreach ($values as $item => $value) {
+				if ($value == 0) {
+					continue;
+				}
+
 				$angleend += (int) (360 * $value / $sum) + 1;
 				$angleend = ($angleend > 360) ? 360 : $angleend;
 
@@ -615,9 +641,15 @@ class CPieGraphDraw extends CGraphDraw {
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -653,12 +685,15 @@ class CPieGraphDraw extends CGraphDraw {
 	}
 
 	public function draw() {
-		$start_time = microtime(true);
+		$debug_mode = CWebUser::getDebugMode();
+		if ($debug_mode) {
+			$start_time = microtime(true);
+		}
 		set_image_header();
+		$this->calculateTopPadding();
 
 		$this->selectData();
 
-		$this->shiftY = 30;
 		$this->shiftYLegend = 20;
 		$this->shiftXleft = 10;
 		$this->shiftXright = 0;
@@ -671,16 +706,32 @@ class CPieGraphDraw extends CGraphDraw {
 
 		if ($this->drawLegend == 1) {
 			$this->sizeX -= $this->shiftXleft + $this->shiftXright + $this->shiftlegendright;
-			$this->sizeY -= $this->shiftY + $this->shiftYLegend + 12 * $this->num + 8;
+			$this->sizeY -= $this->shiftY + $this->shiftYLegend + 14 * $this->num + 8;
 		}
-		else {
+		elseif ($this->with_vertical_padding) {
 			$this->sizeX -= $this->shiftXleft * 2;
 			$this->sizeY -= $this->shiftY * 2;
 		}
 
+		if (!$this->with_vertical_padding) {
+			if ($this->drawLegend == 1) {
+				// Increase size of graph by sum of: 8px legend font size and 5px legend item bottom shift.
+				$this->sizeY += 13;
+			}
+			else {
+				// Remove y shift if only graph is rendered (no labels, header, vertical paddings).
+				$this->shiftY = 0;
+			}
+		}
+
 		$this->sizeX = min($this->sizeX, $this->sizeY);
 		$this->sizeY = min($this->sizeX, $this->sizeY);
 
+		if ($this->sizeX + $this->shiftXleft > $this->fullSizeX) {
+			$this->sizeX = $this->fullSizeX - $this->shiftXleft - $this->shiftXleft;
+			$this->sizeY = min($this->sizeX, $this->sizeY);
+		}
+
 		$this->calc3dheight($this->sizeY);
 
 		$this->exploderad = (int) $this->sizeX / 100;
@@ -697,7 +748,7 @@ class CPieGraphDraw extends CGraphDraw {
 		$this->drawHeader();
 
 		// for each metric
-		$values = array();
+		$values = [];
 		for ($i = 0; $i < $this->num; $i++) {
 			$type = $this->items[$i]['calc_type'];
 
@@ -741,23 +792,22 @@ class CPieGraphDraw extends CGraphDraw {
 				$this->drawElementPie($values);
 		}
 
-		$this->drawLogo();
 		if ($this->drawLegend == 1) {
 			$this->drawLegend();
 		}
 
-		$str = sprintf('%0.2f', microtime(true) - $start_time);
-		$str = _s('Data from %1$s. Generated in %2$s sec.', $this->dataFrom, $str);
-		$strSize = imageTextSize(6, 0, $str);
-		imageText(
-			$this->im,
-			6,
-			0,
-			$this->fullSizeX - $strSize['width'] - 5,
-			$this->fullSizeY - 5,
-			$this->getColor('Gray'),
-			$str
-		);
+		if ($debug_mode) {
+			$str = sprintf('%0.2f', microtime(true) - $start_time);
+			imageText(
+				$this->im,
+				6,
+				90,
+				$this->fullSizeX - 2,
+				$this->fullSizeY - 5,
+				$this->getColor('Gray'),
+				_s('Data from %1$s. Generated in %2$s sec.', $this->dataFrom, $str)
+			);
+		}
 
 		unset($this->items, $this->data);
 
