@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -22,12 +22,12 @@
 class CFormList extends CList {
 
 	protected $editable = true;
-	protected $formInputs = ['ctextbox', 'cnumericbox', 'ctextarea', 'ccombobox', 'ccheckbox', 'cpassbox', 'cipbox'];
+	protected $formInputs = ['ctextbox', 'cnumericbox', 'ctextarea', 'ccombobox', 'ccheckbox', 'cpassbox'];
 
 	public function __construct($id = null) {
 		parent::__construct();
 
-		$this->addClass('table-forms');
+		$this->addClass(ZBX_STYLE_TABLE_FORMS);
 
 		if ($id) {
 			$this->setId(zbx_formatDomId($id));
@@ -46,7 +46,9 @@ class CFormList extends CList {
 			$input_class = strtolower(get_class($input));
 
 			if (in_array($input_class, $this->formInputs)) {
-				$input_id = $input->getAttribute('id');
+				if ($input_class !== 'ccheckbox' || $input->getLabel() === '') {
+					$input_id = $input->getAttribute('id');
+				}
 			}
 		}
 
@@ -71,7 +73,7 @@ class CFormList extends CList {
 	public function addInfo($text) {
 		$this->addItem(
 			[
-				(new CDiv(_('Info')))->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
+				(new CDiv())->addClass(ZBX_STYLE_TABLE_FORMS_TD_LEFT),
 				(new CDiv(
 					(new CDiv($text))->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 				))->addClass(ZBX_STYLE_TABLE_FORMS_TD_RIGHT)
