@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -965,7 +965,7 @@ class C20XmlValidator {
 
 				case SCREEN_RESOURCE_MAP:
 				case SCREEN_RESOURCE_SCREEN:
-				case SCREEN_RESOURCE_TRIGGERS_OVERVIEW:
+				case SCREEN_RESOURCE_TRIGGER_OVERVIEW:
 				case SCREEN_RESOURCE_DATA_OVERVIEW:
 					$rules = ['type' => XML_ARRAY, 'rules' => [
 						'name' =>			['type' => XML_STRING | XML_REQUIRED]
