@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,23 +27,22 @@ class CScreenScreen extends CScreenBase {
 	 * @return CDiv (screen inside container)
 	 */
 	public function get() {
-		$screen = API::Screen()->get(array(
+		$screen = API::Screen()->get([
 			'screenids' => $this->screenitem['resourceid'],
 			'output' => API_OUTPUT_EXTEND,
 			'selectScreenItems' => API_OUTPUT_EXTEND
-		));
+		]);
 		$screen = reset($screen);
 
-		$screenBuilder = new CScreenBuilder(array(
+		$screenBuilder = new CScreenBuilder([
 			'isFlickerfree' => $this->isFlickerfree,
 			'mode' => ($this->mode == SCREEN_MODE_EDIT || $this->mode == SCREEN_MODE_SLIDESHOW) ? SCREEN_MODE_SLIDESHOW : SCREEN_MODE_PREVIEW,
 			'timestamp' => $this->timestamp,
 			'screen' => $screen,
-			'period' => $this->timeline['period'],
-			'stime' => $this->timeline['stimeNow'],
 			'profileIdx' => $this->profileIdx,
-			'updateProfile' => false
-		));
+			'from' => $this->timeline['from'],
+			'to' => $this->timeline['to']
+		]);
 
 		return $this->getOutput($screenBuilder->show(), true);
 	}
