@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -281,257 +281,6 @@ $hostList->addRow(_('Enabled'),
 	(new CCheckBox('status', HOST_STATUS_MONITORED))->setChecked($data['status'] == HOST_STATUS_MONITORED)
 );
 
-if ($data['clone_hostid'] != 0) {
-	// host applications
-	$hostApps = API::Application()->get([
-		'output' => ['name'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false,
-		'preservekeys' => true,
-		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
-	]);
-
-	if ($hostApps) {
-		$applicationsList = [];
-		foreach ($hostApps as $hostAppId => $hostApp) {
-			$applicationsList[$hostAppId] = $hostApp['name'];
-		}
-		order_result($applicationsList);
-
-		$listBox = new CListBox('applications', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($applicationsList);
-		$hostList->addRow(_('Applications'), $listBox);
-	}
-
-	// host items
-	$hostItems = API::Item()->get([
-		'output' => ['itemid', 'hostid', 'key_', 'name'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false,
-		'filter' => ['flags' => ZBX_FLAG_DISCOVERY_NORMAL]
-	]);
-
-	if ($hostItems) {
-		$hostItems = CMacrosResolverHelper::resolveItemNames($hostItems);
-
-		$itemsList = [];
-		foreach ($hostItems as $hostItem) {
-			$itemsList[$hostItem['itemid']] = $hostItem['name_expanded'];
-		}
-		order_result($itemsList);
-
-		$listBox = new CListBox('items', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($itemsList);
-		$hostList->addRow(_('Items'), $listBox);
-	}
-
-	// host triggers
-	$hostTriggers = API::Trigger()->get([
-		'output' => ['triggerid', 'description'],
-		'selectItems' => ['type'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false,
-		'filter' => ['flags' => [ZBX_FLAG_DISCOVERY_NORMAL]]
-	]);
-
-	if ($hostTriggers) {
-		$triggersList = [];
-
-		foreach ($hostTriggers as $hostTrigger) {
-			if (httpItemExists($hostTrigger['items'])) {
-				continue;
-			}
-			$triggersList[$hostTrigger['triggerid']] = $hostTrigger['description'];
-		}
-
-		if ($triggersList) {
-			order_result($triggersList);
-
-			$listBox = new CListBox('triggers', null, 8);
-			$listBox->setAttribute('disabled', 'disabled');
-			$listBox->addItems($triggersList);
-			$hostList->addRow(_('Triggers'), $listBox);
-		}
-	}
-
-	// host graphs
-	$hostGraphs = API::Graph()->get([
-		'output' => ['graphid', 'name'],
-		'selectHosts' => ['hostid'],
-		'selectItems' => ['type'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false,
-		'filter' => ['flags' => [ZBX_FLAG_DISCOVERY_NORMAL]]
-	]);
-
-	if ($hostGraphs) {
-		$graphsList = [];
-		foreach ($hostGraphs as $hostGraph) {
-			if (count($hostGraph['hosts']) > 1) {
-				continue;
-			}
-			if (httpItemExists($hostGraph['items'])) {
-				continue;
-			}
-			$graphsList[$hostGraph['graphid']] = $hostGraph['name'];
-		}
-
-		if ($graphsList) {
-			order_result($graphsList);
-
-			$listBox = new CListBox('graphs', null, 8);
-			$listBox->setAttribute('disabled', 'disabled');
-			$listBox->addItems($graphsList);
-			$hostList->addRow(_('Graphs'), $listBox);
-		}
-	}
-
-	// discovery rules
-	$hostDiscoveryRuleIds = [];
-
-	$hostDiscoveryRules = API::DiscoveryRule()->get([
-		'output' => ['itemid', 'hostid', 'key_', 'name'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false
-	]);
-
-	if ($hostDiscoveryRules) {
-		$hostDiscoveryRules = CMacrosResolverHelper::resolveItemNames($hostDiscoveryRules);
-
-		$discoveryRuleList = [];
-		foreach ($hostDiscoveryRules as $discoveryRule) {
-			$discoveryRuleList[$discoveryRule['itemid']] = $discoveryRule['name_expanded'];
-		}
-		order_result($discoveryRuleList);
-		$hostDiscoveryRuleIds = array_keys($discoveryRuleList);
-
-		$listBox = new CListBox('discoveryRules', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($discoveryRuleList);
-		$hostList->addRow(_('Discovery rules'), $listBox);
-	}
-
-	// item prototypes
-	$hostItemPrototypes = API::ItemPrototype()->get([
-		'output' => ['itemid', 'hostid', 'key_', 'name'],
-		'hostids' => [$data['clone_hostid']],
-		'discoveryids' => $hostDiscoveryRuleIds,
-		'inherited' => false
-	]);
-
-	if ($hostItemPrototypes) {
-		$hostItemPrototypes = CMacrosResolverHelper::resolveItemNames($hostItemPrototypes);
-
-		$prototypeList = [];
-		foreach ($hostItemPrototypes as $itemPrototype) {
-			$prototypeList[$itemPrototype['itemid']] = $itemPrototype['name_expanded'];
-		}
-		order_result($prototypeList);
-
-		$listBox = new CListBox('itemsPrototypes', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($prototypeList);
-		$hostList->addRow(_('Item prototypes'), $listBox);
-	}
-
-	// Trigger prototypes
-	$hostTriggerPrototypes = API::TriggerPrototype()->get([
-		'output' => ['triggerid', 'description'],
-		'selectItems' => ['type'],
-		'hostids' => [$data['clone_hostid']],
-		'discoveryids' => $hostDiscoveryRuleIds,
-		'inherited' => false
-	]);
-
-	if ($hostTriggerPrototypes) {
-		$prototypeList = [];
-		foreach ($hostTriggerPrototypes as $triggerPrototype) {
-			// skip trigger prototypes with web items
-			if (httpItemExists($triggerPrototype['items'])) {
-				continue;
-			}
-			$prototypeList[$triggerPrototype['triggerid']] = $triggerPrototype['description'];
-		}
-
-		if ($prototypeList) {
-			order_result($prototypeList);
-
-			$listBox = new CListBox('triggerprototypes', null, 8);
-			$listBox->setAttribute('disabled', 'disabled');
-			$listBox->addItems($prototypeList);
-			$hostList->addRow(_('Trigger prototypes'), $listBox);
-		}
-	}
-
-	// Graph prototypes
-	$hostGraphPrototypes = API::GraphPrototype()->get([
-		'output' => ['graphid', 'name'],
-		'selectHosts' => ['hostid'],
-		'hostids' => [$data['clone_hostid']],
-		'discoveryids' => $hostDiscoveryRuleIds,
-		'inherited' => false
-	]);
-
-	if ($hostGraphPrototypes) {
-		$prototypeList = [];
-		foreach ($hostGraphPrototypes as $graphPrototype) {
-			if (count($graphPrototype['hosts']) == 1) {
-				$prototypeList[$graphPrototype['graphid']] = $graphPrototype['name'];
-			}
-		}
-		order_result($prototypeList);
-
-		$listBox = new CListBox('graphPrototypes', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($prototypeList);
-		$hostList->addRow(_('Graph prototypes'), $listBox);
-	}
-
-	// host prototypes
-	$hostPrototypes = API::HostPrototype()->get([
-		'output' => ['hostid', 'name'],
-		'discoveryids' => $hostDiscoveryRuleIds,
-		'inherited' => false
-	]);
-
-	if ($hostPrototypes) {
-		$prototypeList = [];
-		foreach ($hostPrototypes as $hostPrototype) {
-			$prototypeList[$hostPrototype['hostid']] = $hostPrototype['name'];
-		}
-		order_result($prototypeList);
-
-		$listBox = new CListBox('hostPrototypes', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($prototypeList);
-		$hostList->addRow(_('Host prototypes'), $listBox);
-	}
-
-	// web scenarios
-	$httpTests = API::HttpTest()->get([
-		'output' => ['httptestid', 'name'],
-		'hostids' => [$data['clone_hostid']],
-		'inherited' => false
-	]);
-
-	if ($httpTests) {
-		$httpTestList = [];
-
-		foreach ($httpTests as $httpTest) {
-			$httpTestList[$httpTest['httptestid']] = $httpTest['name'];
-		}
-
-		order_result($httpTestList);
-
-		$listBox = new CListBox('httpTests', null, 8);
-		$listBox->setAttribute('disabled', 'disabled');
-		$listBox->addItems($httpTestList);
-		$hostList->addRow(_('Web scenarios'), $listBox);
-	}
-}
-
 $divTabs->addTab('hostTab', _('Host'), $hostList);
 
 // templates
