@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,15 +19,6 @@
 **/
 
 
-$serviceWidget = new CWidget(null, 'service-list');
-$serviceWidget->addPageHeader(_('CONFIGURATION OF IT SERVICES'), SPACE);
-$serviceWidget->addHeader(_('IT services'));
-
-// create form
-$serviceForm = new CForm();
-$serviceForm->setName('serviceForm');
-
-$serviceWidget->addItem(BR());
-$serviceWidget->addItem($this->data['tree']->getHTML());
-
-return $serviceWidget;
+return (new CWidget())
+	->setTitle(_('Services'))
+	->addItem($this->data['tree']->getHTML());
