@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -311,10 +311,10 @@ elseif (hasRequest('add') || hasRequest('update')) {
 
 			// copy template screens
 			$dbTemplateScreens = API::TemplateScreen()->get([
+				'noInheritance' => true,
 				'output' => ['screenid'],
 				'templateids' => $cloneTemplateId,
-				'preservekeys' => true,
-				'inherited' => false
+				'preservekeys' => true
 			]);
 
 			if ($dbTemplateScreens) {
@@ -620,7 +620,7 @@ else {
 	// Select writable templates:
 	$linked_template_ids = [];
 	$writable_templates = [];
-	$linked_hosts_ids = [];
+	$linked_hostids = [];
 	$writable_hosts = [];
 	foreach ($templates as $template) {
 		$linked_template_ids = array_merge(
@@ -630,8 +630,8 @@ else {
 			zbx_objectValues($template['hosts'], 'hostid')
 		);
 
-		$linked_hosts_ids = array_merge(
-			$linked_hosts_ids,
+		$linked_hostids = array_merge(
+			$linked_hostids,
 			zbx_objectValues($template['hosts'], 'hostid')
 		);
 	}
@@ -644,11 +644,11 @@ else {
 			'preservekeys' => true
 		]);
 	}
-	if ($linked_hosts_ids) {
-		$linked_hosts_ids = array_unique($linked_hosts_ids);
+	if ($linked_hostids) {
+		$linked_hostids = array_unique($linked_hostids);
 		$writable_hosts = API::Host()->get([
 			'output' => ['hostid'],
-			'hostsids' => $linked_hosts_ids,
+			'hostids' => $linked_hostids,
 			'editable' => true,
 			'preservekeys' => true
 		]);
