@@ -24,12 +24,12 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"25 January 2021"
+#define ZABBIX_REVDATE		"22 February 2021"
 #define ZABBIX_VERSION_MAJOR	4
 #define ZABBIX_VERSION_MINOR	0
-#define ZABBIX_VERSION_PATCH	28
+#define ZABBIX_VERSION_PATCH	29
 #ifndef ZABBIX_VERSION_REVISION
-#	define ZABBIX_VERSION_REVISION	f537e31544
+#	define ZABBIX_VERSION_REVISION	aea59da84c
 #endif
 #ifdef _WINDOWS
 #	ifndef ZABBIX_VERSION_RC_NUM
