@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -269,7 +269,8 @@ int	byte_value_from_proc_file(FILE *f, const char *label, const char *guard, zbx
 	if (NULL != guard)
 	{
 		guard_len = strlen(guard);
-		pos = ftell(f);
+		if (0 > (pos = ftell(f)))
+			return FAIL;
 	}
 
 	while (NULL != fgets(buf, (int)sizeof(buf), f))
@@ -278,11 +279,16 @@ int	byte_value_from_proc_file(FILE *f, const char *label, const char *guard, zbx
 		{
 			if (0 == strncmp(buf, guard, guard_len))
 			{
-				fseek(f, pos, SEEK_SET);
+				if (0 != fseek(f, pos, SEEK_SET))
+					ret = FAIL;
 				break;
 			}
 
-			pos = ftell(f);
+			if (0 > (pos = ftell(f)))
+			{
+				ret = FAIL;
+				break;
+			}
 		}
 
 		if (0 != strncmp(buf, label, label_len))
@@ -1173,7 +1179,7 @@ void	zbx_proc_get_process_stats(zbx_procstat_util_t *procs, int procs_num)
 static zbx_sysinfo_proc_t	*proc_create(int pid, unsigned int flags)
 {
 	char			*procname = NULL, *cmdline = NULL, *name_arg0 = NULL;
-	uid_t			uid = -1;
+	uid_t			uid = (uid_t)-1;
 	zbx_sysinfo_proc_t	*proc = NULL;
 	int			ret = FAIL;
 	size_t			cmdline_nbytes;
