@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,39 +23,63 @@
 #include "../common/common.h"
 #include "log.h"
 
-static int	get_kstat_named_field(const char *name, const char *field, kstat_named_t *returned_data, char **error)
+static int	get_kstat_named_field(const char *name, const char *field, zbx_uint64_t *field_value, char **error)
 {
-	int		ret = FAIL;
+	int		ret = FAIL, min_instance = -1;
 	kstat_ctl_t	*kc;
-	kstat_t		*kp;
+	kstat_t		*kp, *min_kp;
 	kstat_named_t	*kn;
 
 	if (NULL == (kc = kstat_open()))
 	{
-		*error = zbx_dsprintf(*error, "Cannot open kernel statistics facility: %s", zbx_strerror(errno));
+		*error = zbx_dsprintf(NULL, "Cannot open kernel statistics facility: %s", zbx_strerror(errno));
 		return FAIL;
 	}
 
-	if (NULL == (kp = kstat_lookup(kc, NULL, -1, (char *)name)))
+	for (kp = kc->kc_chain; NULL != kp; kp = kp->ks_next)	/* traverse all kstat chain */
 	{
-		*error = zbx_dsprintf(*error, "Cannot look up in kernel statistics facility: %s", zbx_strerror(errno));
+		if (0 != strcmp(name, kp->ks_name))		/* network interface name */
+			continue;
+
+		if (0 != strcmp("net", kp->ks_class))
+			continue;
+
+		/* find instance with the smallest number */
+
+		if (-1 == min_instance || kp->ks_instance < min_instance)
+		{
+			min_instance = kp->ks_instance;
+			min_kp = kp;
+		}
+
+		if (0 == min_instance)
+			break;
+	}
+
+	if (-1 != min_instance)
+		kp = min_kp;
+
+	if (NULL == kp)
+	{
+		*error = zbx_dsprintf(NULL, "Cannot look up interface \"%s\" in kernel statistics facility", name);
 		goto clean;
 	}
 
 	if (-1 == kstat_read(kc, kp, 0))
 	{
-		*error = zbx_dsprintf(*error, "Cannot read from kernel statistics facility: %s", zbx_strerror(errno));
+		*error = zbx_dsprintf(NULL, "Cannot read from kernel statistics facility: %s", zbx_strerror(errno));
 		goto clean;
 	}
 
 	if (NULL == (kn = (kstat_named_t *)kstat_data_lookup(kp, (char *)field)))
 	{
-		*error = zbx_dsprintf(*error, "Cannot look up data in kernel statistics facility: %s",
+		*error = zbx_dsprintf(NULL, "Cannot look up data in kernel statistics facility: %s",
 				zbx_strerror(errno));
 		goto clean;
 	}
 
-	*returned_data = *kn;
+	*field_value = get_kstat_numeric_value(kn);
+
 	ret = SUCCEED;
 clean:
 	kstat_close(kc);
@@ -65,199 +89,187 @@ clean:
 
 static int	NET_IF_IN_BYTES(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "rbytes64", &kn, &error) ||
-			SUCCEED == get_kstat_named_field(if_name, "rbytes", &kn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "rbytes64", &value, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else
+	else if (0 == value && SUCCEED != get_kstat_named_field(if_name, "rbytes", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_IN_PACKETS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "ipackets64", &kn, &error) ||
-			SUCCEED == get_kstat_named_field(if_name, "ipackets", &kn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "ipackets64", &value, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else
+	else if (0 == value && SUCCEED != get_kstat_named_field(if_name, "ipackets", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_IN_ERRORS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "ierrors", &kn, &error))
-	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
-	}
-	else
+	if (SUCCEED != get_kstat_named_field(if_name, "ierrors", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
+	SET_UI64_RESULT(result, value);
+
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_OUT_BYTES(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "obytes64", &kn, &error) ||
-			SUCCEED == get_kstat_named_field(if_name, "obytes", &kn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "obytes64", &value, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else
+	else if (0 == value && SUCCEED != get_kstat_named_field(if_name, "obytes", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_OUT_PACKETS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "opackets64", &kn, &error) ||
-			SUCCEED == get_kstat_named_field(if_name, "opackets", &kn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "opackets64", &value, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else
+	else if (0 == value && SUCCEED != get_kstat_named_field(if_name, "opackets", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_OUT_ERRORS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*error = NULL;
+	zbx_uint64_t	value;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "oerrors", &kn, &error))
-	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
-	}
-	else
+	if (SUCCEED != get_kstat_named_field(if_name, "oerrors", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
+	SET_UI64_RESULT(result, value);
+
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_TOTAL_BYTES(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	ikn, okn;
-	char		*error = NULL;
+	zbx_uint64_t	value_in, value_out;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "rbytes64", &ikn, &error) &&
-			SUCCEED == get_kstat_named_field(if_name, "obytes64", &okn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "rbytes64", &value_in, &error)
+			|| SUCCEED != get_kstat_named_field(if_name, "obytes64", &value_out, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&ikn) + get_kstat_numeric_value(&okn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else if (SUCCEED == get_kstat_named_field(if_name, "rbytes", &ikn, &error) &&
-			SUCCEED == get_kstat_named_field(if_name, "obytes", &okn, &error))
-	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&ikn) + get_kstat_numeric_value(&okn));
-	}
-	else
+	else if ((0 == value_in && SUCCEED != get_kstat_named_field(if_name, "rbytes", &value_in, &error)) ||
+			(0 == value_out && SUCCEED != get_kstat_named_field(if_name, "obytes", &value_out, &error)))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value_in + value_out);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_TOTAL_PACKETS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	ikn, okn;
-	char		*error = NULL;
+	zbx_uint64_t	value_in, value_out;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "ipackets64", &ikn, &error) &&
-			SUCCEED == get_kstat_named_field(if_name, "opackets64", &okn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "ipackets64", &value_in, &error)
+			|| SUCCEED != get_kstat_named_field(if_name, "opackets64", &value_out, &error))
 	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&ikn) + get_kstat_numeric_value(&okn));
+		SET_MSG_RESULT(result, error);
+		return SYSINFO_RET_FAIL;
 	}
-	else if (SUCCEED == get_kstat_named_field(if_name, "ipackets", &ikn, &error) &&
-			SUCCEED == get_kstat_named_field(if_name, "opackets", &okn, &error))
-	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&ikn) + get_kstat_numeric_value(&okn));
-	}
-	else
+	else if ((0 == value_in && SUCCEED != get_kstat_named_field(if_name, "ipackets", &value_in, &error)) ||
+			(0 == value_out && SUCCEED != get_kstat_named_field(if_name, "opackets", &value_out, &error)))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	zbx_free(error);
+	SET_UI64_RESULT(result, value_in + value_out);
 
 	return SYSINFO_RET_OK;
 }
 
 static int	NET_IF_TOTAL_ERRORS(const char *if_name, AGENT_RESULT *result)
 {
-	kstat_named_t	ikn, okn;
-	char		*error = NULL;
+	zbx_uint64_t	value_in, value_out;
+	char		*error;
 
-	if (SUCCEED == get_kstat_named_field(if_name, "ierrors", &ikn, &error) &&
-			SUCCEED == get_kstat_named_field(if_name, "oerrors", &okn, &error))
-	{
-		SET_UI64_RESULT(result, get_kstat_numeric_value(&ikn) + get_kstat_numeric_value(&okn));
-	}
-	else
+	if (SUCCEED != get_kstat_named_field(if_name, "ierrors", &value_in, &error) ||
+			SUCCEED != get_kstat_named_field(if_name, "oerrors", &value_out, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
+	SET_UI64_RESULT(result, value_in + value_out);
+
 	return SYSINFO_RET_OK;
 }
 
 int	NET_IF_COLLISIONS(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	kstat_named_t	kn;
-	char		*if_name, *error = NULL;
+	zbx_uint64_t	value;
+	char		*if_name, *error;
 
 	if (1 < request->nparam)
 	{
@@ -273,13 +285,13 @@ int	NET_IF_COLLISIONS(AGENT_REQUEST *request, AGENT_RESULT *result)
 		return SYSINFO_RET_FAIL;
 	}
 
-	if (SUCCEED != get_kstat_named_field(if_name, "collisions", &kn, &error))
+	if (SUCCEED != get_kstat_named_field(if_name, "collisions", &value, &error))
 	{
 		SET_MSG_RESULT(result, error);
 		return SYSINFO_RET_FAIL;
 	}
 
-	SET_UI64_RESULT(result, get_kstat_numeric_value(&kn));
+	SET_UI64_RESULT(result, value);
 
 	return SYSINFO_RET_OK;
 }
@@ -401,7 +413,7 @@ int	NET_IF_OUT(AGENT_REQUEST *request, AGENT_RESULT *result)
 		return SYSINFO_RET_FAIL;
 	}
 
-	if (NULL == mode || '\0' == *mode || 0 ==strcmp(mode, "bytes"))
+	if (NULL == mode || '\0' == *mode || 0 == strcmp(mode, "bytes"))
 		ret = NET_IF_OUT_BYTES(if_name, result);
 	else if (0 == strcmp(mode, "packets"))
 		ret = NET_IF_OUT_PACKETS(if_name, result);
@@ -436,7 +448,7 @@ int	NET_IF_TOTAL(AGENT_REQUEST *request, AGENT_RESULT *result)
 		return SYSINFO_RET_FAIL;
 	}
 
-	if (NULL == mode || '\0' == *mode || 0 ==strcmp(mode, "bytes"))
+	if (NULL == mode || '\0' == *mode || 0 == strcmp(mode, "bytes"))
 		ret = NET_IF_TOTAL_BYTES(if_name, result);
 	else if (0 == strcmp(mode, "packets"))
 		ret = NET_IF_TOTAL_PACKETS(if_name, result);
